﻿class EquationGrapher extends MovieClip {
	function EquationGrapher() {
		evaluator = new EquationEvaluator();
		drawer = createEmptyMovieClip("drawer", 200);
		//drawer at depth 200;
		//grid at depth 100;
	}
	var drawer:MovieClip;
	var evaluator:EquationEvaluator;
	var eqObject:EquationObject;
	var equation = "";
	var canvasW = 320;
	var canvasH = 320;
	var accuracy = 2;
	var yZoom;
	var xZoom;
	//two pixels for unit;
	var xMin = -10;
	var xMax = 10;
	var yMin = null;
	var yMax = null;
	var custom_yMin;
	var custom_yMax;
	var yArray = [];
	var xArray = [];
	//#########################################
	function samplePoints() {
		yMin = null;
		yMax = null;
		var nSample = int(canvasW/accuracy);
		var range = xMax-xMin;
		var step = range/nSample;
		var ev = evaluator;
		//var x = xMin;
		yArray = [];
		xArray = [];
		for (var k = 0; k<=nSample; k++) {
			var x = xMin+step*k;
			ev.setVariable("x", x);
			var y = Number(ev);
			yArray.push(y);
			xArray.push(x);
			//x += step;
			var dy = y-y0;
			//-----------------------------
			if (k>1) {
				var turn = Math.abs(dy/dy0);
				if (isFinite(turn)) {
					if (turn<0.02) {
						yArray[k-2] = null;
						trace("infinity when x="+(x-2*step));
					}
					if (turn>50) {
						yArray[k] = null;
						trace("infinity when x="+x);
					}
				}
			}
			var y0 = y;
			var dy0 = dy;
		}
		//----------------------------------------------
		//calculate yMax and yMin;
		for (var k = 1; k<yArray.length; k++) {
			var v = yArray[k];
			if (isFinite(v)) {
				if (v>yMax || yMax == null) {
					yMax = v;
				}
				if (v<yMin || yMin == null) {
					yMin = v;
				}
			}
		}
	}
	//########################################
	function samplePointsCropped() {
		yMin = custom_yMin;
		yMax = custom_yMax;
		var nSample = int(canvasW/accuracy);
		var range = xMax-xMin;
		var step = range/nSample;
		var ev = evaluator;
		//var x = xMin;
		yArray = [];
		xArray = [];
		var upBound = false;
		var downBound = false;
		for (var k = 0; k<=nSample; k++) {
			var x = xMin+step*k;
			ev.setVariable("x", x);
			var y = Number(ev);
			//-----------------------------
			if (y>yMax) {
				if (upBound) {
					y = Infinity;
				} else {
					y = yMax;
				}
				upBound = true;
			} else {
				upBound = false;
			}
			//---------------------------
			if (y<yMin) {
				if (downBound) {
					y = -Infinity;
				} else {
					y = yMin;
				}
				downBound = true;
			} else {
				downBound = false;
			}
			//------------------------------------
			yArray.push(y);
			xArray.push(x);
		}
	}
	//-----------------------------------------------------------------
	function render() {
		if (custom_yMin && custom_yMax) {
			samplePointsCropped();
		} else {
			samplePoints();
		}
		var yRange = yMax-yMin;
		//for "sin(x)^2-cos(x)^2", the range is 0, we can not zoom it for fit;
		if (!yRange>0 || yRange<0.000001) {
			yRange = 10;
			var y = yMax;
			yMax = y+5;
			yMin = y-5;
		}
		yZoom = canvasH/yRange;
		xZoom = canvasW/(xMax-xMin);
		//trace("yArray="+yArray);
		//trace("xArray="+xArray);
		//trace("----------------"+yMin+","+yMax);
		//plot yArray
		drawer.clear();
		drawer.lineStyle(1, 0xFF0000, 100);
		var x = 0;
		var skip = true;
		for (var i = 0; i<yArray.length; i++) {
			var y = yArray[i];
			if (!isFinite(y)) {
				skip = true;
			} else {
				if (skip) {
					//trace("moveTo "+x+","+(y-yMin)*zoom);
					drawer.moveTo(x, graphToLocal(0, y).y);
					skip = false;
				} else {
					//trace("lineTo "+x+","+(y-yMin)*zoom);
					drawer.lineTo(x, graphToLocal(0, y).y);
					skip = false;
				}
			}
			x += accuracy;
		}
	}
	//#########################################
	function setEquationObject(obj) {
		eqObject = obj;
		equation = eqObject.equation;
		evaluator.setEquationObject(obj);
	}
	function setAccuracy(numPixels) {
		accuracy = numPixels;
	}
	function setXRange(xMin2, xMax2) {
		if (xMin2 == xMax2) {
			trace("Range of X can not be 0");
			return;
		}
		xMin = Math.min(xMin2, xMax2);
		xMax = Math.max(xMax2, xMin2);
	}
	function getXYRange() {
		return {yMin:yMin, yMax:yMax, xMin:xMin, xMax:xMax};
	}
	function localToGraph(x, y) {
		var gx = (x/xZoom)+xMin;
		var gy = (-y/yZoom)+yMin;
		return {x:gx, y:gy};
	}
	function graphToLocal(gx, gy) {
		var x = (gx-xMin)*xZoom;
		var y = -(gy-yMin)*yZoom;
		return {x:x, y:y};
	}
	function setEquation(str) {
		setEquationObject(new EquationObject(str));
	}
	function setCanvasSize(w, h) {
		canvasW = w;
		canvasH = h;
	}
	//######################################
	function setYRange(yMin2, yMax2) {
		custom_yMin = Math.min(yMin2, yMax2);
		custom_yMax = Math.max(yMax2, yMin2);
	}
}
